/******************************************************************
*******************************************************************
***																***
***		TS for DT400 Top-module                 			***
***																***
*******************************************************************
******************************************************************/

								/*************************************/
								/**	Module definition   			**/
								/*************************************/
module cap_sts( PULL_UP,
	FX2_IFCLK, FX2_FIFOADR, FX2_FD, FX2_SLRD, FX2_SLWR, FX2_SLOE, FX2_PKTEND,
	TS_SCLK_T, TS_PBVAL_T, TS_SD_T,
	TS_SCLK_S, TS_PBVAL_S, TS_SD_S,
	START, TERRESTRIAL,
	BCAS_IO,
	IR, U2_DIR, U2_A,
	PLED_RED_OFF, POWER_ON );
								/*************************************/
								/**	I/Os definitions				**/
								/*************************************/
		/* To avoid the warning in synthesis */
	input  PULL_UP ;				// pulled-up input for fix-'1'
		/* signals between FX2 */
	input  FX2_IFCLK;
	output [1:0] FX2_FIFOADR;
	output [7:0] FX2_FD;
	output FX2_SLRD;
	output FX2_SLWR;
	output FX2_SLOE;
	output FX2_PKTEND;
		/* signals from tuner module(TS data) */
	input  TS_SCLK_T;
	input  TS_PBVAL_T;
	input  TS_SD_T;
	input  TS_SCLK_S;
	input  TS_PBVAL_S;
	input  TS_SD_S;
		/* signals to control CPLD */
	input  START;
	input  TERRESTRIAL;
	input  BCAS_IO;
		/* remote-control & power sense signals */
	input  IR ;
	output U2_DIR ;
	output U2_A ;
	input  PLED_RED_OFF ;
	output POWER_ON ;
								/*************************************/
								/**	fixed value ONE/ZERO	 		**/
								/**	 to avoid warning in synthesis  **/
								/*************************************/
	wire ONE ;
	ALT_INBUF pullup_inbuf ( .i(PULL_UP), .o(ONE));
	defparam pullup_inbuf.weak_pull_up_resistor = "on";
	wire ZERO = ~ONE ;
								/*************************************/
								/**	serial->parallel data from tuner**/
								/*************************************/
	wire [15:0] ts_data_t, ts_data_s;
	wire ts_req_t, ts_req_s ;
	wire ts_req_clr ;
	ser2par ser2par_t(~TS_SCLK_T, TS_SD_T, TS_PBVAL_T, ts_data_t, ts_req_t, ts_req_clr) ;
	ser2par ser2par_s(~TS_SCLK_S, TS_SD_S, TS_PBVAL_S, ts_data_s, ts_req_s, ts_req_clr) ;
	wire [15:0] ts_data = TERRESTRIAL ? ts_data_t : ts_data_s ;
	wire ts_req = TERRESTRIAL ? ts_req_t : ts_req_s ;

	wire [7:0] bcas_data;
	wire bcas_req;
	wire bcas_req_clr;
	bcas_ser2par bcas_ser2par(FX2_IFCLK, BCAS_IO, bcas_data, bcas_req, bcas_req_clr);
								/*************************************/
								/**	sequencer putting data to FIFO  **/
								/*************************************/
	reg slwr_reg;
	reg ts_clr_reg;
	reg bcas_clr_reg;
	reg [1:0] ts_bcas_sw;
	reg [1:0] _wait;
	reg _ts_req;
	reg _bcas_req;

	always@ (posedge FX2_IFCLK) begin
		_ts_req <= ts_req;
		if ( ( _ts_req == 1 ) && ( _wait == 0 ) ) begin
			ts_clr_reg <= 1;
			ts_bcas_sw <= 0;
			_wait <= 1;
		end else if ( ( bcas_req == 1 ) && ( _wait == 0 ) ) begin
			bcas_clr_reg <= 1;
			ts_bcas_sw <= 1;
			_wait <= 1;
		end
		if ( _wait == 1 ) begin
			slwr_reg <= 0;
			if ( ts_bcas_sw == 0 ) 
				_wait <= 2;
			else
				_wait <= 3;
		end
		if(_wait == 2) begin
			_wait <= 3;
		end
		if(_wait == 3) begin
			slwr_reg <= 1;
			ts_clr_reg <= 0;
			bcas_clr_reg <= 0;
			_wait <= 0;
		end
	end

	assign ts_req_clr = ts_clr_reg;
	assign bcas_req_clr = bcas_clr_reg;
								/*************************************/
								/**	Interface signals between FX2  	**/
								/*************************************/
	assign FX2_SLOE = ONE ;
	assign FX2_FIFOADR = (ts_bcas_sw == 0) ? 2'b10 : 2'b01;
	assign FX2_PKTEND = ONE ;
	assign FX2_SLRD = ONE ;
	assign FX2_SLWR = (START == 1) ? slwr_reg : 1'b1;
	assign FX2_FD = (ts_bcas_sw == 1) ? bcas_data :
					(_wait == 2) ? ts_data[7:0] : ts_data[15:8];
								/*************************************/
								/**	Remote control & power sense	**/
								/*************************************/
	assign U2_DIR = IR ? 1'b0 : 1'b1 ;
	assign U2_A = IR ? 1'bz : 1'b0 ;
	assign POWER_ON = PLED_RED_OFF ;

endmodule
