#include "StdAfx.h"
#include "EcmDat.h"
#include "SharedEcmQueue.h"

CSharedEcmQueue::CSharedEcmQueue(DWORD dwMaxCount)
	: m_hMutex(NULL), m_hEvent(NULL), m_hMapping(NULL), m_pView(NULL), m_dwSize(0)
{
	m_hMutex = ::CreateMutex(NULL, FALSE, SHARED_ECM_QUEUE_MUTEX_NAME);
	m_hEvent = ::CreateEvent(NULL, FALSE, FALSE, SHARED_ECM_QUEUE_EVENT_NAME);

	DWORD dwSize = DATA_OFFSET + ECM_DAT_SIZE * dwMaxCount;

	this->Lock();
	m_hMapping = ::CreateFileMapping(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE, 0, dwSize, SHARED_ECM_QUEUE_NAME);
	BOOL bAlreadyExists = (::GetLastError() == ERROR_ALREADY_EXISTS);
	if (m_hMapping != NULL) {
		m_pView = ::MapViewOfFile(m_hMapping, FILE_MAP_ALL_ACCESS, 0, 0, 0);
		if (m_pView != NULL) {
			this->m_dwSize = dwSize;
			if (!bAlreadyExists) {
				::ZeroMemory(m_pView, dwSize);
				this->setMaxCount(dwMaxCount);
				this->setCount(0);
				this->setFrontIndex(0);
				this->setBackIndex(dwMaxCount - 1);
			}
		}
	}
	this->Unlock();

	if (!m_hMutex || !m_hEvent || !m_hMapping || !m_pView) {
		this->deleteAll();
	}
}

CSharedEcmQueue::~CSharedEcmQueue() {
	this->deleteAll();
}

void CSharedEcmQueue::deleteAll() {
	if (m_pView) { ::UnmapViewOfFile(m_pView); m_pView = NULL; }
	if (m_hMapping) { ::CloseHandle(m_hMapping); m_hMapping = NULL; }
	if (m_hEvent) { ::CloseHandle(m_hEvent); m_hEvent = NULL; }
	if (m_hMutex) { ::CloseHandle(m_hMutex); m_hMutex = NULL; }
	m_dwSize = 0;
}

// ********************************************************************************

void CSharedEcmQueue::push_front(const ECM_DAT& ecm) {
	this->Lock();
	DWORD dwCount = this->getCount();
	if (dwCount >= this->getMaxCount()) {
		this->pop_back();
		dwCount--;
	}
	DWORD dwFrontIndex = this->decFrontIndex();
	this->setEcm(dwFrontIndex, ecm);
	this->setCount(++dwCount);
	this->Unlock();
}

void CSharedEcmQueue::push_back(const ECM_DAT& ecm) {
	this->Lock();
	DWORD dwCount = this->getCount();
	if (dwCount >= this->getMaxCount()) {
		this->pop_front();
		dwCount--;
	}
	DWORD dwBackIndex = this->incBackIndex();
	this->setEcm(dwBackIndex, ecm);
	this->setCount(++dwCount);
	this->Unlock();
}

void CSharedEcmQueue::pop_front() {
	this->Lock();
	DWORD dwCount = this->getCount();
	if (dwCount > 0) {
		this->setCount(--dwCount);
		this->incFrontIndex();
	}
	this->Unlock();
}

void CSharedEcmQueue::pop_back() {
	this->Lock();
	DWORD dwCount = this->getCount();
	if (dwCount > 0) {
		this->setCount(--dwCount);
		this->decBackIndex();
	}
	this->Unlock();
}

const ECM_DAT CSharedEcmQueue::operator [] (DWORD dwIndex) {
	this->Lock();
	DWORD dwNewIndex = this->getFrontIndex() + dwIndex;
	DWORD dwMaxCount = this->getMaxCount();
	if (dwNewIndex >= dwMaxCount) {
		dwNewIndex -= dwMaxCount;
	}
	ECM_DAT ecm = this->getEcm(dwNewIndex);
	this->Unlock();
	return ecm;
}

// ********************************************************************************

void CSharedEcmQueue::setEcm(DWORD dwIndex, const ECM_DAT& ecm) {
	DWORD dwByteIndex = this->calcEcmByteIndex(dwIndex);
	this->setBYTEs(dwByteIndex, ecm.key, sizeof(ecm.key));
	this->setDWORD(dwByteIndex + sizeof(ecm.key), ecm.magic);
	this->setDWORD(dwByteIndex + sizeof(ecm.key) + sizeof(ecm.magic), ecm.time);
}

ECM_DAT CSharedEcmQueue::getEcm(DWORD dwIndex) {
	ECM_DAT ecm;
	DWORD dwByteIndex = this->calcEcmByteIndex(dwIndex);
	this->getBYTEs(dwByteIndex, ecm.key, sizeof(ecm.key));
	ecm.magic = this->getDWORD(dwByteIndex + sizeof(ecm.key));
	ecm.time = this->getDWORD(dwByteIndex + sizeof(ecm.key) + sizeof(ecm.magic));
	return ecm;
}

// ********************************************************************************

void CSharedEcmQueue::setMaxCount(DWORD dwMaxCount) {
	this->setDWORD(MAX_COUNT_OFFSET, dwMaxCount);
}

DWORD CSharedEcmQueue::getMaxCount() {
	return this->getDWORD(MAX_COUNT_OFFSET);
}

void CSharedEcmQueue::setCount(DWORD dwCount) {
	this->setDWORD(COUNT_OFFSET, dwCount);
}

DWORD CSharedEcmQueue::getCount() {
	return this->getDWORD(COUNT_OFFSET);
}

void CSharedEcmQueue::setFrontIndex(DWORD dwFrontIndex) {
	this->setDWORD(FRONT_INDEX_OFFSET, dwFrontIndex);
}

DWORD CSharedEcmQueue::getFrontIndex() {
	return this->getDWORD(FRONT_INDEX_OFFSET);
}

void CSharedEcmQueue::setBackIndex(DWORD dwBackIndex) {
	this->setDWORD(BACK_INDEX_OFFSET, dwBackIndex);
}

DWORD CSharedEcmQueue::getBackIndex() {
	return this->getDWORD(BACK_INDEX_OFFSET);
}

// ********************************************************************************

DWORD CSharedEcmQueue::incFrontIndex() {
	DWORD dwFrontIndex = this->getFrontIndex();
	if (dwFrontIndex < this->getMaxCount() - 1) {
		dwFrontIndex++;
	}
	else {
		dwFrontIndex = 0;
	}
	this->setFrontIndex(dwFrontIndex);
	return dwFrontIndex;
}

DWORD CSharedEcmQueue::decFrontIndex() {
	DWORD dwFrontIndex = this->getFrontIndex();
	if (dwFrontIndex > 0) {
		dwFrontIndex--;
	}
	else {
		dwFrontIndex = this->getMaxCount() - 1;
	}
	this->setFrontIndex(dwFrontIndex);
	return dwFrontIndex;
}

DWORD CSharedEcmQueue::incBackIndex() {
	DWORD dwBackIndex = this->getBackIndex();
	if (dwBackIndex < this->getMaxCount() - 1) {
		dwBackIndex++;
	}
	else {
		dwBackIndex = 0;
	}
	this->setBackIndex(dwBackIndex);
	return dwBackIndex;
}

DWORD CSharedEcmQueue::decBackIndex() {
	DWORD dwBackIndex = this->getBackIndex();
	if (dwBackIndex > 0) {
		dwBackIndex--;
	}
	else {
		dwBackIndex = this->getMaxCount() - 1;
	}
	this->setBackIndex(dwBackIndex);
	return dwBackIndex;
}

// ********************************************************************************

/*
BYTE CSharedEcmQueue::getBYTE(DWORD dwByteIndex) {
	return *this->getBYTEPointer(dwByteIndex);
}

void CSharedEcmQueue::setBYTE(DWORD dwByteIndex, const BYTE data) {
	*this->getBYTEPointer(dwByteIndex) = data;
}

WORD CSharedEcmQueue::getWORD(DWORD dwByteIndex) {
	return *((WORD*)this->getBYTEPointer(dwByteIndex));
}

void CSharedEcmQueue::setWORD(DWORD dwByteIndex, const WORD data) {
	*((WORD*)this->getBYTEPointer(dwByteIndex)) = data;
}
*/
DWORD CSharedEcmQueue::getDWORD(DWORD dwByteIndex) {
	return *((DWORD*)this->getBYTEPointer(dwByteIndex));
}

void CSharedEcmQueue::setDWORD(DWORD dwByteIndex, const DWORD data) {
	*((DWORD*)this->getBYTEPointer(dwByteIndex)) = data;
}

void CSharedEcmQueue::getBYTEs(DWORD dwByteIndex, BYTE* pDst, DWORD dwSize) {
	BYTE* pSrc = this->getBYTEPointer(dwByteIndex);
	memcpy(pDst, pSrc, dwSize);
}

void CSharedEcmQueue::setBYTEs(DWORD dwByteIndex, const BYTE* pSrc, DWORD dwSize) {
	BYTE* pDst = this->getBYTEPointer(dwByteIndex);
	memcpy(pDst, pSrc, dwSize);
}

BYTE* CSharedEcmQueue::getBYTEPointer(DWORD dwByteIndex) {
	return (BYTE*)m_pView + dwByteIndex;
}
