#pragma once

//#include "EcmDat.h"
#include "stdafx.h"

#define SHARED_ECM_QUEUE_NAME _T("SHARED_ECM_QUEUE")
#define SHARED_ECM_QUEUE_MUTEX_NAME (SHARED_ECM_QUEUE_NAME _T("_MUTEX"))
#define SHARED_ECM_QUEUE_EVENT_NAME (SHARED_ECM_QUEUE_NAME _T("_EVENT"))

#define SHARED_ECM_QUEUE_MAX_COUNT (100)
#define SHARED_ECM_QUEUE_GET_RETRY_COUNT (2)
#define SHARED_ECM_QUEUE_GET_RETRY_TIME (300)

#define ECM_DAT_SIZE (sizeof(((ECM_DAT*)NULL)->key) + sizeof(((ECM_DAT*)NULL)->magic) + sizeof(((ECM_DAT*)NULL)->time))
#define ECM_SHARED_QUEUE_NAME _T("ECM_SHARED_QUEUE")

#define MAX_COUNT_OFFSET (0)
#define COUNT_OFFSET (MAX_COUNT_OFFSET + sizeof(DWORD))
#define FRONT_INDEX_OFFSET (COUNT_OFFSET + sizeof(DWORD))
#define BACK_INDEX_OFFSET (FRONT_INDEX_OFFSET + sizeof(DWORD))
#define DATA_OFFSET (BACK_INDEX_OFFSET + sizeof(DWORD)) // ECM Data start address

class CSharedEcmQueue
{
public:
	CSharedEcmQueue(DWORD dwMaxCount);
	~CSharedEcmQueue();

public:
	void push_front(const ECM_DAT& ecm);
	void push_back(const ECM_DAT& ecm);
	void pop_front();
	void pop_back();
//	inline DWORD size() { this->Lock(); DWORD result = this->getCount(); this->Unlock(); return result; }
	inline DWORD size() { return this->getCount(); }
	const ECM_DAT operator [] (DWORD dwIndex); // getter only
	inline BOOL Lock(DWORD dwMilliseconds = INFINITE) {
		return (::WaitForSingleObject(m_hMutex, dwMilliseconds) == WAIT_OBJECT_0);
	}
	inline BOOL Unlock() { return ::ReleaseMutex(m_hMutex); }
	inline BOOL WaitEvent(DWORD dwMilliseconds = INFINITE) {
		return (::WaitForSingleObject(m_hEvent, dwMilliseconds) == WAIT_OBJECT_0);
	}
	inline BOOL SetEvent() { return ::SetEvent(m_hEvent); }
	inline BOOL ResetEvent() { return ::ResetEvent(m_hEvent); }

private:
	HANDLE m_hMutex;
	HANDLE m_hEvent;
	HANDLE m_hMapping;
	void* m_pView;
	DWORD m_dwSize;

private:
	void setEcm(DWORD dwIndex, const ECM_DAT& ecm);
	ECM_DAT getEcm(DWORD dwIndex);
	inline DWORD calcEcmByteIndex(DWORD dwIndex) {
		return DATA_OFFSET + ECM_DAT_SIZE * dwIndex;
	}

	inline void setMaxCount(DWORD dwMaxCount);
	inline DWORD getMaxCount();
	inline void setCount(DWORD dwCount);
	inline DWORD getCount();
	inline void setFrontIndex(DWORD dwFrontIndex);
	inline DWORD getFrontIndex();
	inline void setBackIndex(DWORD dwBackIndex);
	inline DWORD getBackIndex();

	inline DWORD incFrontIndex();
	inline DWORD decFrontIndex();
	inline DWORD incBackIndex();
	inline DWORD decBackIndex();

/*
	inline BYTE getBYTE(DWORD dwByteIndex);
	inline void setBYTE(DWORD dwByteIndex, const BYTE data);
	inline WORD getWORD(DWORD dwByteIndex);
	inline void setWORD(DWORD dwByteIndex, const WORD data);
*/
	inline DWORD getDWORD(DWORD dwByteIndex);
	inline void setDWORD(DWORD dwByteIndex, const DWORD data);
	inline void getBYTEs(DWORD dwByteIndex, BYTE* pDst, DWORD dwSize);
	inline void setBYTEs(DWORD dwByteIndex, const BYTE* pSrc, DWORD dwSize);

	inline BYTE* getBYTEPointer(DWORD dwByteIndex);

	void deleteAll();
};
